'use client';

import { EmailIcon, StaffIcon, WorkIcon } from '@/components/icons';
import {
	CompletedAssesment,
	JobDetails,
	Loader,
	PersonalDetails,
	ReviewStatus,
} from '@/components/others/CommonComponents';
import { Tabs, TabsContent, TabsList, TabsTrigger } from '@/components/ui/tabs';
import {
	ApplicantTabsEnum,
	URL_SEARCH_PARAMS,
} from '@/components/utils/constants';
import {
	fetchEndpointData,
	updateResourceEndpointData,
} from '@/components/utils/server-functions';
import {
	Application,
	TApplicantFeedback,
	TApplicantStatus,
} from '@/components/utils/types';
import { useQueryCustomWrapper } from '@/utils';
import { useParams, usePathname, useRouter } from 'next/navigation';
import React from 'react';
import { useState } from 'react';
import { useQuery } from 'react-query';
import { ToastContainer, toast } from 'react-toastify';
import 'react-toastify/dist/ReactToastify.css';

export default function () {
	const [errMsg, setErrMsg] = useState<string>('');
	const [loading, setLoading] = useState(false);
	const { applicantId } = useParams();
	const pathname = usePathname();
	const router = useRouter();

	const [activeTab, setActiveTab] = useState<ApplicantTabsEnum>(
		ApplicantTabsEnum.application_details
	);

	const {
		isLoading: applicantIsLoading,
		isError: applicantIsError,
		data: applicationInfo,
	} = useQuery({
		queryFn: useQueryCustomWrapper<Application>,
		queryKey: [
			`applicant-data-${applicantId}`,
			{
				url: `applications/${applicantId}`,
				qFunc: fetchEndpointData,
				options: {
					populate: {
						user: true,
						files: true,
						job: {
							populate: {
								ministry: true,
								job_type: true,
							},
						},
					},
				},
			},
		],
		enabled: !!applicantId,
	});

	const [feedback, setFeedback] = useState<TApplicantFeedback>({
		status: applicationInfo?.data.status,
		comment: '',
	});

	const handleFeedback = ({ cb }: { cb: () => void }) => {
		if (!feedback) return;
		setLoading(true);

		updateResourceEndpointData({
			url: `applications/${applicantId}`,
			id: `${applicantId}`,
			data: {
				...feedback,
				user: applicationInfo?.data.user?.id,
			},
		})
			.then(({ err }) => {
				if (err) {
					if (err.status === 400) setErrMsg(err.message);
					else if (err.status === 401)
						router.push(
							`/?${URL_SEARCH_PARAMS.redirect}=${encodeURIComponent(pathname)}`
						);
					else if (err.status === 403) {
						toast.error('Permission Denied', { position: 'top-right' });
						setErrMsg('Permission denied');
					} else {
						toast.error('Soemthing went wrong', { position: 'top-right' });

						setErrMsg('Something went wrong');
					}
				} else {
					toast.success('Successfully updated the record', {
						position: 'top-right',
					});

					cb();
				}
			})
			.finally(() => setLoading(false));
	};

	return (
		<div className=' h-full flex flex-col pl-[15px] pr-[15px] pt-[16px] pb-[33px] gap-y-[32px] md:gap-y-0 md:gap-[20px] md:px-[16px]'>
			{applicantIsLoading ? (
				<div className='md:col-span-6 rounded-[12px] bg-white shadow-sidebarShadow h-[500px] w-full'>
					<Loader />
				</div>
			) : applicantIsError ? (
				<div className='md:col-span-6 rounded-[12px] bg-white shadow-sidebarShadow h-[500px] w-full'>
					<Loader
						{...{
							title: 'Error loading data',
						}}
					/>
				</div>
			) : (
				<React.Fragment>
					<JobDetails {...{ job: applicationInfo?.data.job }} />

					<div className='w-[70%] p-[20px] py-[16px] mx-auto shadow-applicantBoxDetailsShadow rounded-[8px]'>
						<Tabs value={activeTab} className='w-full'>
							<TabsList className='border-b-[1px] px-[8px] h-[52px] w-full justify-start gap-[10px]'>
								<TabsTrigger
									value={ApplicantTabsEnum.application_details}
									className='flex flex-col gap-[4px]'>
									<StaffIcon
										{...{
											svgElementClassName: 'stroke-inherit fill-inherit',
											applyToSvgEl: true,
											styles: {
												width: '24px',
												height: '24px',
											},
										}}
									/>
									<p className='text-inherit font-medium text-[16px] leading-[22px] text-center'>
										Application Details
									</p>
								</TabsTrigger>

								<TabsTrigger
									value={ApplicantTabsEnum.review_status}
									className='flex flex-col gap-[4px]'>
									<WorkIcon
										{...{
											svgElementClassName: 'stroke-inherit fill-inherit',
											applyToSvgEl: true,
											styles: {
												width: '24px',
												height: '24px',
											},
										}}
									/>
									<p className='text-inherit font-medium text-[16px] leading-[22px] text-center'>
										Review status
									</p>
								</TabsTrigger>

								<TabsTrigger
									value={ApplicantTabsEnum.complete_assesment}
									className='flex flex-col gap-[4px]'>
									<EmailIcon
										{...{
											svgElementClassName: 'stroke-inherit fill-inherit',
											applyToSvgEl: false,
											styles: {
												width: '24px',
												height: '24px',
											},
										}}
									/>
									<p className='text-inherit font-medium text-[16px] leading-[22px] text-center'>
										Complete Assessment
									</p>
								</TabsTrigger>
							</TabsList>

							<TabsContent value={ApplicantTabsEnum.application_details}>
								<PersonalDetails
									onClick={() => {
										setActiveTab(ApplicantTabsEnum.review_status);
									}}
									{...{
										user: applicationInfo?.data.user!,
										files: applicationInfo?.data.files,
									}}
								/>
							</TabsContent>

							<TabsContent value={ApplicantTabsEnum.review_status}>
								<ReviewStatus
									{...{
										handleNext: () =>
											handleFeedback({
												cb: () =>
													setActiveTab(ApplicantTabsEnum.complete_assesment),
											}),
										handlePrevious: () =>
											setActiveTab(ApplicantTabsEnum.application_details),
										savedComment: applicationInfo?.data.comment,
										canGoToNext:
											applicationInfo?.data.status !==
												TApplicantStatus.Awarded &&
											applicationInfo?.data.status !== TApplicantStatus.Rejected
												? true
												: false,
										status: applicationInfo?.data.status,
										selectedStatus: feedback.status,
										comment: feedback.comment,
										handleChange: setFeedback,
										errMsg,
										loading,
									}}
								/>
							</TabsContent>

							<TabsContent value={ApplicantTabsEnum.complete_assesment}>
								<CompletedAssesment
									{...{
										user: applicationInfo?.data.user!,
									}}
								/>
							</TabsContent>
						</Tabs>
					</div>
					<ToastContainer />
				</React.Fragment>
			)}
		</div>
	);
}
